
EPSILON = Math::PI / 1000999888 # a small number

class Single_hand_random_iterator

  def initialize(top, how_many)
    srand
    @top = top.to_f
    @divider = how_many - 1
    @nth = -1
  end

  # the function that does the work
  def another
    @nth += 1
    return nil if @nth > @divider

    hand = rand() * @top
    return hand
  end  # another

end  # iterator


# random just returns random hand values
#
class Double_hand_random_iterator

  def initialize(top, how_many)

    srand

    @top = top.to_f
    @divider = how_many - 1
    @nth_1 = -1     # first incr puts it at 0
    @nth_2 = 0
  end

  # the function that does the work
  def another

    @nth_1 += 1
    if @nth_1 > @divider
      @nth_1 = 0
      @nth_2 += 1
      return nil if @nth_2 > @divider
    end

    h1 = rand() * @top

    h2 = rand() * @top

    return [h1, h2]

  end  # another

end  # iterator

class Single_hand_cycle_iterator

  def initialize(top, how_many)
    @top = top.to_f
    @divider = how_many - 1
    @nth = -1
  end

  def another

    @nth += 1
    return nil if @nth > @divider

    hand = (@top * @nth) / @divider
    # Note: no tie avoidance. It is assumed that the
    # two hands will have different how_many's

    return hand

  end  # another

end

# cycle samples the whole range evenly
#
class Double_hand_cycle_iterator

  def initialize(top, how_many)
    @top = top.to_f
    @divider = how_many - 1
    @nth_1 = -1    # first incr puts it at 0
    @nth_2 = 0
  end

  # the function that does the work
  def another

    @nth_1 += 1

    if @nth_1 > @divider
      @nth_1 = 0
      @nth_2 += 1
      return nil if @nth_2 > @divider
    end

    h1 = (@top * @nth_1) / @divider

    # avoid ties
    if @nth_1 == @nth_2
      if @nth_1.even? and @nth_1 < @divider
        h1 += EPSILON * @top
      else
        h1 -= EPSILON * @top
      end
    end

    h2 = (@top * @nth_2) / @divider

    return [h1, h2]

  end # another

end  # iterator

